# Generated by Django 2.0.7 on 2018-07-10 11:05

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        ('rayures', '0002_orderreturn'),
    ]

    operations = [
        migrations.CreateModel(
            name='RayureMeta',
            fields=[
                ('id', models.CharField(max_length=255, primary_key=True, serialize=False)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('deleted_at', models.DateTimeField(null=True)),
                ('idempotency_key', models.TextField(null=True)),
                ('request_id', models.TextField(null=True)),
                ('source', models.TextField(null=True)),
                ('content_type', models.ForeignKey(limit_choices_to=models.Q(app_label='rayures'), on_delete=django.db.models.deletion.CASCADE, to='contenttypes.ContentType')),
                ('event', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, to='rayures.Event')),
            ],
        ),
        migrations.RemoveField(
            model_name='rayuresmeta',
            name='content_type',
        ),
        migrations.RemoveField(
            model_name='rayuresmeta',
            name='event',
        ),
        migrations.DeleteModel(
            name='RayuresMeta',
        ),
    ]
